#include <math.h>

struct ENGINE_ZBUFFER { int width; int height; float * depth; };

void engine_3d_find_rotation(struct ENGINE_VECTOR_3D * rot,float x, float y, float z) {
	float distance = sqrtf(x * x + y * y);
	rot->x = atan2f(y,x) * (180.0/M_PI);
	rot->y = atan2f(z,distance) * (180.0/M_PI);
	rot->z = 0.0;
}

void engine_3d_rotate_position(struct ENGINE_VECTOR_3D * pos,float x, float y, float z, float rx, float ry, float rz) {
	rx = rx * (M_PI/180.0);
	ry = ry * (M_PI/180.0);
	rz = rz * (M_PI/180.0);
	float sinX = sinf(rx);
	float cosX = cosf(rx);
	float sinY = sinf(ry);
	float cosY = cosf(ry);
	float sinZ = sinf(rz);
	float cosZ = cosf(rz);
	pos->x = x * (cosY * cosZ) + y * (sinX * sinY * cosZ - cosX * sinZ) + z * (cosX * sinY * cosZ + sinX * sinZ);
	pos->y = x * (cosY * sinZ) + y * (sinX * sinY * sinZ + cosX * cosZ) + z * (cosX * sinY * sinZ - sinX * cosZ);
	pos->z = x * (-sinY) + y * (sinX * cosY) + z * (cosX * cosY);
}

void engine_3d_project(struct ENGINE_VECTOR_2D * screen,float x, float y, float z, float rx, float ry, float rz, float fovHor, float fovVert) {
	struct ENGINE_VECTOR_3D pos;
	pos.x = x;
	pos.y = y;
	pos.z = z;
	
	engine_3d_rotate_position(&pos,pos.x,pos.y,pos.z,0.0,ry,0.0);
	engine_3d_rotate_position(&pos,pos.x,pos.y,pos.z,0.0,0.0,rx);
	pos.x = -pos.x; pos.y = -pos.y; pos.z = -pos.z;
	struct ENGINE_VECTOR_3D rot;
	engine_3d_find_rotation(&rot,pos.x,pos.y,pos.z);
	
	// rot.y (screen X)
	while (rot.y >  180.0) { rot.y -= 360.0; }
	while (rot.y < -180.0) { rot.y += 360.0; }
	rot.y = rot.y / (fovHor * 0.5);
	
	// rot.x (screen Y)
	while (rot.x >  180.0) { rot.x -= 360.0; }
	while (rot.x < -180.0) { rot.x += 360.0; }
	rot.x = rot.x / (fovVert * 0.5);
	
	screen->x = rot.y;
	screen->y = rot.x;
}

/*
// simple projection
void engine_3d_rendertarget_draw_quad(
	struct ENGINE_TEXTURE* texture, // Texture
	struct ENGINE_ZBUFFER* zbuffer, // Depth buffer (not used)
	float tsx, float tsy, // Texture start
	float tex, float tey, // Texture end
	float wx1, float wy1, float wz1, // Corner 1
	float wx2, float wy2, float wz2, // Corner 2
	float wx3, float wy3, float wz3, // Corner 3
	float wx4, float wy4, float wz4, // Corner 4
	float rx, float ry, float rz, // Camera rotation
	float fovHor, float fovVert, // Camera FoV
	char writeToZBuffer // Whether to write to depth buffer (not used)
) {
	// Projecting the corners
	struct ENGINE_VECTOR_2D corners[4];
	engine_3d_project(&corners[0], wx1, wy1, wz1, rx, ry, rz, fovHor, fovVert);
	engine_3d_project(&corners[1], wx2, wy2, wz2, rx, ry, rz, fovHor, fovVert);
	engine_3d_project(&corners[2], wx3, wy3, wz3, rx, ry, rz, fovHor, fovVert);
	engine_3d_project(&corners[3], wx4, wy4, wz4, rx, ry, rz, fovHor, fovVert);
	
	int width = engine_rendertarget -> width;
	int height = engine_rendertarget -> height;	
	struct ENGINE_VECTOR_TEX texoutput;
	
	// Calculate bounding box for the quad
	int minX = width, maxX = 0, minY = height, maxY = 0;
	int i = 0;
	while (i < 4) {
		int x, y;
		engine_position_2d_to_tex(&texoutput, corners[i].x, corners[i].y, width - 1, height - 1, 0);
		x = texoutput.x;
		y = texoutput.y;
		if (x < minX) minX = x;
		if (x > maxX) maxX = x;
		if (y < minY) minY = y;
		if (y > maxY) maxY = y;
		++i;
	}

	// Clipping the bounding box
	if (minX < 0) minX = 0;
	if (maxX >= width) maxX = width - 1;
	if (minY < 0) minY = 0;
	if (maxY >= height) maxY = height - 1;

	// Draw the quad using bilinear interpolation
	int y = minY;
	while (y < maxY) {
		int x = minX;
		while (x < maxX) {
			// Bilinear interpolation to compute texture coordinates
			float ax = (float)(x - minX) / (float)(maxX - minX);
			float ay = (float)(y - minY) / (float)(maxY - minY);
			float texX = tsx + (tex + 1 - tsx) * ax;
			float texY = tsy + (tey + 1 - tsy) * ay;

			struct ENGINE_COLOR color;
			engine_texture_color_get(&color, texture, floor(texX), floor(texY));
			engine_color_set(color.r, color.g, color.b, color.a);
			engine_rendertarget_draw_point(x, y);
			++x;
		}
		++y;
	}
}
*/

void engine_3d_rendertarget_draw_quad(
	struct ENGINE_TEXTURE * texture,
	struct ENGINE_ZBUFFER * zbuffer,
	float tsx, float tsy,
	float tex, float tey,
	float wx1, float wy1, float wz1,
	float wx2, float wy2, float wz2,
	float wx3, float wy3, float wz3,
	float wx4, float wy4, float wz4,
	float rx, float ry, float rz,
	float fovHor, float fovVert,
	char writeToZBuffer
) {
	struct ENGINE_VECTOR_2D v20, v21, v22, v23;
	engine_3d_project(&v20,wx1,wy1,wz1,rx,ry,rz,fovHor,fovVert);
	engine_3d_project(&v21,wx2,wy2,wz2,rx,ry,rz,fovHor,fovVert);
	engine_3d_project(&v22,wx3,wy3,wz3,rx,ry,rz,fovHor,fovVert);
	engine_3d_project(&v23,wx4,wy4,wz4,rx,ry,rz,fovHor,fovVert);
	
	if (
		(v20.x < -2.0 || v20.x > 2.0 || v20.y < -2.0 || v20.y > 2.0) &&
		(v21.x < -2.0 || v21.x > 2.0 || v21.y < -2.0 || v21.y > 2.0) &&
		(v22.x < -2.0 || v22.x > 2.0 || v22.y < -2.0 || v22.y > 2.0) &&
		(v23.x < -2.0 || v23.x > 2.0 || v23.y < -2.0 || v23.y > 2.0)
	) { // Wall is entirely out of bounds
		return;
	}
	
	tey += 1;
	tex += 1;
	int width = engine_rendertarget -> width;
	int height = engine_rendertarget -> height;
	
	struct ENGINE_VECTOR_TEX texoutput;
	engine_position_2d_to_tex(&texoutput,v20.x,v20.y,width - 1,height - 1,0);
	int ssx0 = texoutput.x;
	int ssy0 = texoutput.y;
	engine_position_2d_to_tex(&texoutput,v21.x,v21.y,width - 1,height - 1,0);
	int sex0 = texoutput.x;
	int sey0 = texoutput.y;
	engine_position_2d_to_tex(&texoutput,v22.x,v22.y,width - 1,height - 1,0);
	int ssx1 = texoutput.x;
	int ssy1 = texoutput.y;
	engine_position_2d_to_tex(&texoutput,v23.x,v23.y,width - 1,height - 1,0);
	int sex1 = texoutput.x;
	int sey1 = texoutput.y;
	
	int div = abs(ssy0 - ssy1);
	if (abs(sey0 - sey1) > div) { div = abs(sey0 - sey1); }
	if (div > height) { div = height; }
	
	struct ENGINE_COLOR color;
	
	int ind = 0;
	while (ind <= div) {
		float vProg = (float)ind / (float)div;
		
		int lsx = ssx0 + (((ssx1 - ssx0) * vProg) + 0.5);
		int lex = sex0 + (((sex1 - sex0) * vProg) + 0.5);
		int lsy = ssy0 + (((ssy1 - ssy0) * vProg) + 0.5);
		int ley = sey0 + (((sey1 - sey0) * vProg) + 0.5);
		
		int ldx =  abs(lex - lsx), lix = lsx < lex ? 1 : -1;
		int ldy = -abs(ley - lsy), liy = lsy < ley ? 1 : -1;
		int lerr = ldx + ldy, le2; // error value e_xy
		int lx = lsx;
		int ly = lsy;
		
		for (;;) { // loop
			float hProg = (float)(lx - lsx) / (float)(lex - lsx);
			engine_texture_color_get(&color,texture,floor(tsx + (tex * hProg)),floor(tsy + (tey * vProg)));
			
			if (
				lx >= 0 && lx < width &&
				ly >= 0 && ly < height
			) {
				engine_color_set(color.r,color.g,color.b,color.a);
				engine_rendertarget_draw_point(lx,ly);
			}
			
			if (lx == lex && ly == ley) break;
			le2 = 2 * lerr;
			if (le2 >= ldy) { lerr += ldy; lx += lix; } // e_xy+e_x > 0
			if (le2 <= ldx) { lerr += ldx; ly += liy; } // e_xy+e_y < 0
		}
		
		ind += 1;
	}
}