struct ENGINE_TEXTURE * engine_rendertarget_get() {
	return engine_rendertarget;
}

void engine_rendertarget_fill(int sx,int sy,int ex,int ey) {
	ex += 1; ey += 1;
	int y = sy;
	while (y < ey) {
		int x = sx;
		while (x < ex) {
			engine_rendertarget_draw_point(x,y);
			x += 1;
		}
		y += 1;
	}
}

void engine_rendertarget_draw_texture(struct ENGINE_TEXTURE * texture,int x,int y) {
	int sx = 0;
	if (x < 0) {
		sx -= x;
	}
	if (sx > texture->width) { return; } // unnecessary? maybe faster with a lot of offscreen textures?
	
	int sy = 0;
	if (y < 0) {
		sy -= y;
	}
	if (sy > texture->height) { return; } // unnecessary? maybe faster with a lot of offscreen textures?
	
	int ex = texture->width;
	if (x + ex > engine_rendertarget->width) {
		ex -= (x + ex) - engine_rendertarget->width;
		if (ex < 0) { return; } // unnecessary? maybe faster with a lot of offscreen textures?
	}
	
	int ey = texture->height;
	if (y + ey > engine_rendertarget->height) {
		ey -= (y + ey) - engine_rendertarget->height;
		if (ey < 0) { return; } // unnecessary? maybe faster with a lot of offscreen textures?
	}
	
	struct ENGINE_COLOR color;
	int tx = sx;
	while (sy < ey) {
		while (tx < ex) {
			engine_texture_color_get(&color,texture,tx,sy);
			engine_color_set(color.r,color.g,color.b,color.a);
			engine_rendertarget_draw_point(x + tx,y + sy);
			++tx;
		}
		tx = sx;
		++sy;
	}
}

void engine_rendertarget_draw_file(char * fpath) {
	unsigned char buffer[engine_rendertarget->width * engine_rendertarget->height * 4];
	FILE * f = fopen(fpath,"r");
	fread(buffer,engine_rendertarget->width * engine_rendertarget->height * 4,1,f);
	fclose(f);
	int x = 0;
	int y = 0;
	int colorindex = 0;
	while (y < engine_rendertarget->height) {
		while (x < engine_rendertarget->width) {
			colorindex = (x + (y * engine_rendertarget->width)) * 4;
			engine_color_set(buffer[colorindex],buffer[colorindex+1],buffer[colorindex+2],buffer[colorindex+3]);
			engine_rendertarget_draw_point(x,y);
			x += 1;
		}
		x = 0;
		y += 1;
	}
}

void engine_position_2d_to_tex(struct ENGINE_VECTOR_TEX * pos, float x, float y, int width, int height, char clamp) {
	width = width - 1;
	height = height - 1;
	pos->x = round(((float)width*0.5) + (((float)width*x) * 0.5));
	pos->y = round(((float)height*0.5) + (((float)height*y) * 0.5));
	if (clamp == 1) {
		if (pos->x < 0) { pos->x = 0; }
		if (pos->x > width) { pos->x = width; }
		if (pos->y < 0) { pos->y = 0; }
		if (pos->y > height) { pos->y = height; }
	}
}
