int engine_luaf_lua_event_get_data(lua_State *L) {
	struct ENGINE_EVENT * event = lua_touserdata(L,1);
	char type = event->type;
	if (type == ENGINE_EVENT_TYPE_UNKNOWN) { goto simple; }
	if (type == ENGINE_EVENT_TYPE_NONE) { goto simple; }
	if (type == ENGINE_EVENT_TYPE_EXIT) { goto simple; }
	if (type == ENGINE_EVENT_TYPE_INPUT_KB) {
		struct ENGINE_EVENT_INPUT_KB * data = event->data;
		lua_pushinteger(L,type);
		lua_pushinteger(L,data->key);
		lua_pushinteger(L,data->pressed);
		return 3;
	}
	
	if (type == ENGINE_EVENT_TYPE_INPUT_MOUSE_MOVE) {
		struct ENGINE_EVENT_INPUT_MOUSE_MOVE * data = event->data;
		lua_pushinteger(L,type);
		lua_pushinteger(L,data->x);
		lua_pushinteger(L,data->y);
		lua_pushinteger(L,data->relX);
		lua_pushinteger(L,data->relY);
		return 5;
	}
	
	type = ENGINE_EVENT_TYPE_UNKNOWN;
	simple:
	lua_pushinteger(L,type);
	return 1;
}

int engine_luaf_texture_color_get(lua_State *L) {
	struct ENGINE_TEXTURE * texture = lua_touserdata(L,1);
	int x = luaL_checkinteger(L,2);
	int y = luaL_checkinteger(L,3);
	struct ENGINE_COLOR color;
	engine_texture_color_get(&color,texture,x,y);
	lua_pushinteger(L,color.r);
	lua_pushinteger(L,color.g);
	lua_pushinteger(L,color.b);
	lua_pushinteger(L,color.a);
	return 4;
}

int engine_luaf_window_texture_get(lua_State *L) {
	struct ENGINE_WINDOW * window = lua_touserdata(L,1);
	lua_pushlightuserdata(L,window->texture);
	return 1;
}

int engine_luaf_3d_find_rotation(lua_State *L) {
	float x = luaL_checknumber(L,1);
	float y = luaL_checknumber(L,2);
	float z = luaL_checknumber(L,3);
	
	struct ENGINE_VECTOR_3D rot;
	engine_3d_find_rotation(&rot,x,y,z);
	lua_pushnumber(L,rot.x);
	lua_pushnumber(L,rot.y);
	lua_pushnumber(L,rot.z);
	return 3;
}

int engine_luaf_3d_rotate_position(lua_State *L) {
	float x = luaL_checknumber(L,1);
	float y = luaL_checknumber(L,2);
	float z = luaL_checknumber(L,3);
	float rx = luaL_checknumber(L,4);
	float ry = luaL_checknumber(L,5);
	float rz = luaL_checknumber(L,6);
	
	struct ENGINE_VECTOR_3D pos;
	engine_3d_rotate_position(&pos,x,y,z,rx,ry,rz);
	lua_pushnumber(L,pos.x);
	lua_pushnumber(L,pos.y);
	lua_pushnumber(L,pos.z);
	return 3;
}

int engine_luaf_3d_project(lua_State *L) {
	float x = luaL_checknumber(L,1);
	float y = luaL_checknumber(L,2);
	float z = luaL_checknumber(L,3);
	float rx = luaL_checknumber(L,4);
	float ry = luaL_checknumber(L,5);
	float rz = luaL_checknumber(L,6);
	float fovHor = luaL_checknumber(L,7);
	float fovVert = luaL_checknumber(L,8);
	struct ENGINE_VECTOR_2D pos;
	engine_3d_project(&pos,x,y,z,rx,ry,rz,fovHor,fovVert);
	lua_pushnumber(L,pos.x);
	lua_pushnumber(L,pos.y);
	return 2;
}

int engine_luaf_position_2d_to_tex(lua_State *L) {
	float x = luaL_checknumber(L,1);
	float y = luaL_checknumber(L,2);
	int width = luaL_checknumber(L,3);
	int height = luaL_checknumber(L,4);
	int clamp = luaL_checknumber(L,5);
	struct ENGINE_VECTOR_TEX pos;
	engine_position_2d_to_tex(&pos,x,y,width,height,clamp);
	lua_pushnumber(L,pos.x);
	lua_pushnumber(L,pos.y);
	return 2;
}

void engine_lua_init_manual() {
	lua_pushcfunction(engine_lua_state,engine_luaf_lua_event_get_data);
	lua_setglobal    (engine_lua_state,"engine_lua_event_get_data");
	lua_pushcfunction(engine_lua_state,engine_luaf_texture_color_get);
	lua_setglobal    (engine_lua_state,"engine_texture_color_get");
	lua_pushcfunction(engine_lua_state,engine_luaf_window_texture_get);
	lua_setglobal    (engine_lua_state,"engine_window_texture_get");
	lua_pushcfunction(engine_lua_state,engine_luaf_3d_find_rotation);
	lua_setglobal    (engine_lua_state,"engine_3d_find_rotation");
	lua_pushcfunction(engine_lua_state,engine_luaf_3d_rotate_position);
	lua_setglobal    (engine_lua_state,"engine_3d_rotate_position");
	lua_pushcfunction(engine_lua_state,engine_luaf_3d_project);
	lua_setglobal    (engine_lua_state,"engine_3d_project");
	lua_pushcfunction(engine_lua_state,engine_luaf_position_2d_to_tex);
	lua_setglobal    (engine_lua_state,"engine_position_2d_to_tex");
}
